<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8" />
    <script>
      window.process = {env:{NODE_ENV:''}};
    </script>
    <script>
      window.CUCUMBER_MESSAGES = {{ data }};
      window.TEST_RESULT = {{ testResult }};
     </script>
  </head>
  <body>
    <div id="root"></div>
    <script>
      {{ js }}
    </script>
  </body>
</html>